# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #
"""Check written code with formatters"""
import os

from scargo.scargo_src.sc_src import (
    check_clang_format,
    check_clang_tidy,
    check_copyright,
    check_cppcheck,
    check_cyclomatic,
    check_pragma,
    check_todo,
    prepare_config,
)
from scargo.scargo_src.utils import get_project_root


def scargo_check(
    clang_format: bool,
    clang_tidy: bool,
    copy_right: bool,
    cppcheck: bool,
    cyclomatic: bool,
    pragma: bool,
    todo: bool,
    verbose: bool,
) -> None:
    """
    Check written code using different formatters

    :param bool clang_format: check clang_format
    :param bool clang_tidy: check clang_tidy
    :param bool copy_right:  check copyrights
    :param bool cppcheck: check cpp format
    :param bool cyclomatic: check cyclomatic
    :param bool pragma: check pragma
    :param bool todo: check todo left in code
    :param bool verbose: set verbose
    :return: None
    """
    config = prepare_config()

    # Run all checks by default. If any of checks is specified then do not
    # run all checks; then run only specified checks.
    run_all = not any(
        [
            clang_format,
            clang_tidy,
            copy_right,
            cppcheck,
            cyclomatic,
            pragma,
            todo,
        ]
    )

    # Todo, remove chdir and change cwd for checks
    os.chdir(get_project_root())

    if clang_format or run_all:
        check_clang_format(config, False, verbose)

    if clang_tidy or run_all:
        check_clang_tidy(config, verbose)

    if copy_right or run_all:
        check_copyright(config, False)

    if cppcheck or run_all:
        check_cppcheck()

    if cyclomatic or run_all:
        check_cyclomatic(config)

    if pragma or run_all:
        check_pragma(config, False)

    if todo or run_all:
        check_todo(config)



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

from pathlib import Path

from scargo.jinja.base_gen import BaseGen
from scargo.scargo_src.global_values import SCARGO_PGK_PATH
from scargo.scargo_src.sc_config import ProjectConfig
from scargo.scargo_src.utils import get_project_root


class _ReadmeTemplate(BaseGen):
    """This class is a container for readme md file"""

    def __init__(self):
        self.template_dir = Path(SCARGO_PGK_PATH, "jinja", "templates")
        BaseGen.__init__(self, self.template_dir)

    def generate_readme(self, project_config: ProjectConfig):
        """Generate dirs and files"""
        self.create_file_from_template(
            "README.md.j2",
            get_project_root() / "README.md",
            overwrite=False,
            project=project_config,
        )


def generate_readme(project_config: ProjectConfig):
    readme_template = _ReadmeTemplate()
    readme_template.generate_readme(project_config)



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

import getpass
import os
from dataclasses import dataclass
from pathlib import Path

from scargo.jinja.base_gen import BaseGen
from scargo.scargo_src.global_values import ENV_DEFAULT_NAME, SCARGO_PGK_PATH


@dataclass
class _EnvironmentDescriptor:
    user_name: str = getpass.getuser()
    user_passwd: str = "user"
    user_gid: int = 1000 if os.name == "nt" else os.getgid()
    user_uid: int = 1000 if os.name == "nt" else os.getuid()
    ssh_port: int = 2000
    base_docker_image: str = "ubuntu:20.04"
    conan_username: str = ""
    conan_passwd: str = ""


class _EnvTemplate(BaseGen):
    """
    This class is a container for env file which is used by docker compose
    Is providing basic environmental variable setup
    """

    def __init__(self, output_path: Path):
        self.template_dir = Path(SCARGO_PGK_PATH, "jinja", "docker")
        BaseGen.__init__(self, self.template_dir)
        self.env_output_path = output_path / ENV_DEFAULT_NAME

    def generate_env(self):
        self.create_file_from_template(
            "env.txt.j2",
            self.env_output_path,
            overwrite=False,
            env=_EnvironmentDescriptor(),
        )


def generate_env(output_dir_path: Path):
    env_compose_template = _EnvTemplate(output_dir_path)
    env_compose_template.generate_env()



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #
import sys
from pathlib import Path
from typing import List, Optional

from typer import Argument, Context, Option, Typer

__version__ = "1.0.6"

from scargo.scargo_src.global_values import DESCRIPTION, SCARGO_DEFAULT_CONFIG_FILE
from scargo.scargo_src.sc_build import scargo_build
from scargo.scargo_src.sc_check import scargo_check
from scargo.scargo_src.sc_clean import scargo_clean
from scargo.scargo_src.sc_config import ScargoTargets, Target
from scargo.scargo_src.sc_debug import scargo_debug
from scargo.scargo_src.sc_doc import scargo_doc
from scargo.scargo_src.sc_docker import (
    scargo_docker_build,
    scargo_docker_exec,
    scargo_docker_run,
)
from scargo.scargo_src.sc_fix import scargo_fix
from scargo.scargo_src.sc_flash import scargo_flash
from scargo.scargo_src.sc_gen import scargo_gen
from scargo.scargo_src.sc_logger import get_logger
from scargo.scargo_src.sc_new import scargo_new
from scargo.scargo_src.sc_publish import scargo_publish
from scargo.scargo_src.sc_run import scargo_run
from scargo.scargo_src.sc_test import scargo_test
from scargo.scargo_src.sc_update import scargo_update
from scargo.scargo_src.sc_ver import scargo_version
from scargo.scargo_src.utils import get_config_file_path, get_project_root

###############################################################################


cli = Typer(context_settings=dict(help_option_names=["-h", "--help"]), help=DESCRIPTION)


###############################################################################


@cli.command()
def build(profile: str = Option("Debug", "--profile")):
    """Compile sources."""
    scargo_build(profile)


###############################################################################


@cli.command()
def check(  # pylint: disable=too-many-arguments
    clang_format: bool = Option(False, "--clang-format", help="Run clang-format."),
    clang_tidy: bool = Option(False, "--clang-tidy", help="Run clang-tidy."),
    copy_right: bool = Option(False, "--copyright", help="Run copyright check."),
    cppcheck: bool = Option(False, "--cppcheck", help="Run cppcheck."),
    cyclomatic: bool = Option(False, "--cyclomatic", help="Run python-lizard."),
    pragma: bool = Option(False, "--pragma", help="Run pragma check."),
    todo: bool = Option(False, "--todo", help="Run TODO check."),
    silent: bool = Option(False, "--silent", "-s", help="Show less output."),
):
    """Check source code in directory `src`."""
    scargo_check(
        clang_format,
        clang_tidy,
        copy_right,
        cppcheck,
        cyclomatic,
        pragma,
        todo,
        verbose=not silent,
    )


###############################################################################


@cli.command()
def clean():
    """Remove directory `build`."""
    scargo_clean()


###############################################################################


@cli.command()
def debug(
    bin_path: Optional[Path] = Option(
        None, "--bin", "-b", exists=True, dir_okay=False, help="Path to bin file"
    )
):
    """Use gdb cli to debug"""
    scargo_debug(bin_path)


###############################################################################


@cli.command()
def doc(open_doc: bool = Option(False, "--open", help="Open html documentation")):
    """Create project documentation"""
    scargo_doc(open_doc)


###############################################################################


docker = Typer(help="Manage the docker environment for the project")


@docker.command(
    "build", context_settings={"allow_extra_args": True, "ignore_unknown_options": True}
)
def docker_build(ctx: Context):
    """Build docker layers for this project depending on the target"""
    scargo_docker_build(ctx.args)


@docker.command(
    "run", context_settings={"allow_extra_args": True, "ignore_unknown_options": True}
)
def docker_run(
    ctx: Context,
    command: str = Option(
        "bash",
        "-c",
        "--command",
        metavar="COMMAND",
        help="Select command to be used with docker run.",
    ),
):
    """Run project in docker environment"""
    scargo_docker_run(docker_opts=ctx.args, command=command)


@docker.command(
    "exec", context_settings={"allow_extra_args": True, "ignore_unknown_options": True}
)
def docker_exec(ctx: Context):
    """Attach to existing docker environment"""
    scargo_docker_exec(ctx.args)


cli.add_typer(docker, name="docker")


###############################################################################


@cli.command()
def fix(
    clang_format: bool = Option(
        False, "--clang-format", help="Fix clang-format violations"
    ),
    copy_right: bool = Option(False, "--copyright", help="Fix copyrights violations"),
    pragma: bool = Option(False, "--pragma", help="Fix pragma violations"),
):
    """Fix violations reported by command `check`."""
    scargo_fix(pragma, copy_right, clang_format)


###############################################################################


@cli.command()
def flash(
    app: bool = Option(False, "--app", help="Flash app only"),
    file_system: bool = Option(False, "--fs", help="Flash filesystem only"),
    flash_profile: str = Option(
        "Debug", "--profile", help="Flash base on previously built profile"
    ),
):
    """Flash the target (only available for esp32 for now)."""
    scargo_flash(app, file_system, flash_profile)


###############################################################################


@cli.command()
def gen(  # pylint: disable=too-many-arguments
    profile: str = Option("Debug", "--profile", "-p"),
    gen_ut: Optional[Path] = Option(
        None,
        "--unit-test",
        "-u",
        exists=True,
        help="Generate unit test for chosen file or all headers in directory",
    ),
    gen_mock: Optional[Path] = Option(
        None,
        "--mock",
        "-m",
        exists=True,
        dir_okay=False,
        help="Generate mock of chosen file",
    ),
    certs: Optional[str] = Option(
        None,
        "--certs",
        "-c",
        metavar="<DEVICE ID>",
        help="Generate cert files for azure based on device ID.",
    ),
    file_system: bool = Option(
        False, "--fs", "-f", help="Build the filesystem, base on spiffs dir content."
    ),
    single_bin: bool = Option(
        False, "--bin", "-b", help="Generate single binary image."
    ),
):
    """Manage the auto file generator"""
    project_profile_path = get_project_root() / "build" / profile
    if (gen_ut is gen_mock is certs is None) and not (file_system or single_bin):
        logger = get_logger()
        logger.warning(
            "Please add one of the following options to the command:"
            "\n--unit-test\n--mock\n--certs\n--fs\n--bin"
        )
        sys.exit(1)

    scargo_gen(project_profile_path, gen_ut, gen_mock, certs, file_system, single_bin)


###############################################################################


@cli.command()
def new(  # pylint: disable=too-many-arguments
    name: str,
    bin_name: Optional[str] = Option(
        None,
        "--bin",
        help="Create binary target template.",
        prompt=True,
        prompt_required=False,
    ),
    lib_name: Optional[str] = Option(
        None,
        "--lib",
        help="Create library target template.",
        prompt=True,
        prompt_required=False,
    ),
    target: ScargoTargets = Option("x86", help="Target device."),
    create_docker: bool = Option(
        True, "--docker/--no-docker", help="Initialize docker environment."
    ),
    git: bool = Option(True, "--git/--no-git", help="Initialize git repository."),
):
    """Create new project template."""
    scargo_new(
        name,
        bin_name,
        lib_name,
        Target.get_target_by_id(target.value),
        create_docker,
        git,
    )
    scargo_update(Path(SCARGO_DEFAULT_CONFIG_FILE))


###############################################################################


@cli.command()
def publish(repo: str = Option("", "--repo", "-r", help="Repo name")):
    """Upload conan pkg to repo"""
    scargo_publish(repo)


###############################################################################


@cli.command()
def run(
    bin_path: Optional[Path] = Option(
        None, "--bin", "-b", exists=True, dir_okay=False, help="Path to bin file"
    ),
    profile: str = Option("Debug", "--profile", "-p"),
    skip_build: bool = Option(False, "--skip-build", help="Skip calling scargo build"),
    bin_params: List[str] = Argument(None),
):
    """Build and run project"""
    if not skip_build:
        scargo_build(profile)
    project_profile_path = get_project_root() / "build" / profile
    scargo_run(bin_path, project_profile_path, bin_params)


###############################################################################


@cli.command()
def test(verbose: bool = Option(False, "--verbose", "-v", help="Verbose mode.")):
    """Compile and run all tests in directory `test`."""
    scargo_test(verbose)


###############################################################################


@cli.command()
def update(
    config_file_path: Path = Option(
        SCARGO_DEFAULT_CONFIG_FILE,
        "--config-file",
        "-c",
        exists=True,
        dir_okay=False,
        help="Path to .toml configuration file.",
    ),
):
    """Read .toml config file and generate `CMakeLists.txt`."""
    scargo_update(get_config_file_path(config_file_path))


###############################################################################


@cli.command()
def version():
    """Get scargo version"""
    scargo_version()


###############################################################################

if __name__ == "__main__":
    try:
        cli()
    except Exception as e:
        print(f"\nA fatal error occurred: {e}")
        sys.exit(2)



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

"""Generate cmake for test dir"""
from pathlib import Path
from shutil import copytree

from scargo.jinja.base_gen import BaseGen
from scargo.scargo_src.global_values import SCARGO_PGK_PATH
from scargo.scargo_src.sc_config import Target, TestConfig
from scargo.scargo_src.utils import get_project_root


class _TestsTemplate(BaseGen):
    """Create cmake template in test dir"""

    def __init__(self):
        self.tests_template_dir = Path(SCARGO_PGK_PATH, "jinja", "test_templates")
        BaseGen.__init__(self, self.tests_template_dir)
        project_path = get_project_root()
        self.output_dir = project_path / "tests"

        # List of files to generate once (template_path, output_path)
        self._gen_once_file_list = [
            ("CMakeLists-ut.txt.j2", self.output_dir / "ut" / "CMakeLists.txt"),
            ("CMakeLists-it.txt.j2", self.output_dir / "it" / "CMakeLists.txt"),
            ("CMakeLists-mocks.txt.j2", self.output_dir / "mocks" / "CMakeLists.txt"),
        ]

    def generate_test_dirs_and_files(self, target: Target, tests_config: TestConfig):
        """Generate dirs and files"""
        static_mock_dir = self.output_dir / "mocks" / "static_mock"
        if not static_mock_dir.exists():
            copytree(self.tests_template_dir / "static_mock", static_mock_dir)

        # Update main test cmake on scargo update
        self.create_file_from_template(
            "CMakeLists-test.txt.j2",
            self.output_dir / "CMakeLists.txt",
            overwrite=True,
            target=target,
            tests=tests_config,
        )

        for template, output_path in self._gen_once_file_list:
            self.create_file_from_template(
                template,
                output_path,
                overwrite=False,
                target=target,
                tests=tests_config,
            )


def generate_tests(target: Target, tests_config: TestConfig):
    tests = _TestsTemplate()
    tests.generate_test_dirs_and_files(target, tests_config)



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

from pathlib import Path

from scargo.jinja.base_gen import BaseGen
from scargo.scargo_src.global_values import SCARGO_PGK_PATH
from scargo.scargo_src.sc_config import Config
from scargo.scargo_src.utils import get_project_root


class _ConanTemplate(BaseGen):
    """
    This class is a container for cmake file which is used by cmake.
    """

    def __init__(self):
        template_dir = Path(SCARGO_PGK_PATH, "jinja", "conan")
        BaseGen.__init__(self, template_dir)

    def generate_conanfile(self, config: Config):
        self.create_file_from_template(
            "conanfile.py.j2",
            get_project_root() / "conanfile.py",
            config=config,
        )

    def generate_conanfile_tests(self, config: Config):
        self.create_file_from_template(
            "conanfiletest.j2",
            get_project_root() / "tests/conanfile.py",
            config=config,
        )


def generate_conanfile(config: Config):
    conanfile_template = _ConanTemplate()
    conanfile_template.generate_conanfile(config)
    conanfile_template.generate_conanfile_tests(config)



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

"""Run feature depending on provided args"""
import os
import subprocess
from pathlib import Path
from typing import List, Optional

from scargo.scargo_src.sc_logger import get_logger
from scargo.scargo_src.sc_src import prepare_config


def scargo_run(
    bin_path: Optional[Path], project_profile_path: Path, params: List[str]
) -> None:
    """
    Run command from CLI

    :param str bin_path: path to bin file
    :param Path project_profile_path: path to build file
    :param str params: params for bin file
    :return: None
    """
    logger = get_logger()
    logger.info('Running "%s" build', project_profile_path.name)

    target = prepare_config().project.target
    if "x86" not in target.family:
        logger.info(
            "Run project on x86 architecture is not implemented for %s yet.",
            target.family,
        )
        exit(1)

    if bin_path:
        bin_file_name = bin_path.name
        bin_file_path = bin_path.parent
        try:
            subprocess.check_call([f"./{bin_file_name}"] + params, cwd=bin_file_path)
        except subprocess.CalledProcessError:
            logger.error("bin file not found")
    else:
        bin_dir = project_profile_path / "bin"
        if bin_dir.is_dir():
            bin_list = os.listdir(bin_dir)
            # Run project
            try:
                subprocess.check_call([f"./{bin_list[0]}"] + params, cwd=bin_dir)
            except subprocess.CalledProcessError:
                logger.error("Unable to run bin file")
        else:
            logger.error("Bin file not found")



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

from pathlib import Path

from scargo.jinja.base_gen import BaseGen
from scargo.scargo_src.global_values import SCARGO_PGK_PATH
from scargo.scargo_src.sc_config import ProjectConfig
from scargo.scargo_src.utils import get_project_root


class _CiCdTemplate(BaseGen):
    """This class is a container for CI/CD yaml file which create pipeline"""

    def __init__(self):
        self.template_dir = Path(SCARGO_PGK_PATH, "jinja", "templates")
        BaseGen.__init__(self, self.template_dir)

    def generate_cicd_file(self, project_config: ProjectConfig):
        """Function which creates a cicd yaml file using jinja"""
        self.create_file_from_template(
            ".gitlab-ci.yml.j2",
            get_project_root() / ".gitlab-ci.yml",
            project=project_config,
        )


def generate_cicd(project_config: ProjectConfig):
    cicd_template = _CiCdTemplate()
    cicd_template.generate_cicd_file(project_config)



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

"""Run test from project"""
import subprocess
import sys
from pathlib import Path

from scargo.scargo_src.sc_config import Config
from scargo.scargo_src.sc_logger import get_logger
from scargo.scargo_src.sc_src import prepare_config
from scargo.scargo_src.utils import get_project_root


def scargo_test(verbose: bool) -> None:
    """
    Run test
    :param bool verbose: if verbose
    """
    logger = get_logger()
    config = prepare_config()

    test_dir_name = "tests"
    project_dir = get_project_root()
    tests_src_dir = project_dir / test_dir_name
    test_build_dir = project_dir / "build" / test_dir_name

    if not tests_src_dir.exists():
        logger.error("Directory `tests` does not exist.")
        sys.exit(1)

    if not Path(tests_src_dir, "CMakeLists.txt").exists():
        logger.error("Directory `tests`: File `CMakeLists.txt` does not exist.")
        logger.info("Did you run `scargo update`?")
        sys.exit(1)

    test_build_dir.mkdir(parents=True, exist_ok=True)

    try:
        # Run CMake and build tests.
        subprocess.check_call(
            f"conan install {tests_src_dir} -if {test_build_dir}",
            shell=True,
            cwd=project_dir,
        )
        subprocess.check_call(
            f"conan build {tests_src_dir} -bf {test_build_dir}",
            shell=True,
            cwd=project_dir,
        )
    except subprocess.CalledProcessError:
        logger.error("Failed to build tests.")
        sys.exit(1)

    # run ut
    run_ut(config, verbose, test_build_dir)


def run_ut(config: Config, verbose: bool, cwd: Path) -> None:
    # Run tests.
    cmd = "ctest"

    if verbose:
        cmd += " --verbose"
    try:
        # Using `subprocess.run` because tests can fail,
        # and we do not want Python to throw exception.
        subprocess.run(cmd, shell=True, cwd=cwd)

        # Run code coverage.
        cmd = "gcovr -r ut . --html ut-coverage.html"

        gcov_executable = config.tests.gcov_executable

        if gcov_executable != "":
            cmd = cmd + ' --gcov-executable "' + gcov_executable + '"'

        subprocess.check_call(cmd, shell=True, cwd=cwd)
    except subprocess.CalledProcessError:
        logger = get_logger()
        logger.error("Fail to run project tests")
        sys.exit(1)


def run_it(config: Config, verbose: bool):
    # Run tests.
    cmd = "ctest"

    if verbose:
        cmd += " --verbose"

    subprocess.run(cmd, shell=True)  # Using `subprocess.run` because tests
    # can fail and we do not want Python to
    # throw exception.

    # Run code coverage.
    cmd = "gcovr -r it . --txt it-coverage.txt --html it-coverage.html"

    gcov_executable = config.tests.gcov_executable

    if gcov_executable != "":
        cmd = cmd + ' --gcov-executable "' + gcov_executable + '"'

    subprocess.check_call(cmd, shell=True)
    subprocess.check_call("cat it-coverage.txt", shell=True)



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

import os
import shutil
from pathlib import Path

from scargo import __version__
from scargo.jinja.base_gen import BaseGen
from scargo.scargo_src.global_values import SCARGO_PGK_PATH
from scargo.scargo_src.sc_config import ProjectConfig
from scargo.scargo_src.sc_logger import get_logger
from scargo.scargo_src.utils import get_project_root


class _DockerComposeTemplate(BaseGen):
    """
    This class is a container for docker compose yaml files creation with multilayer approach
    """

    def __init__(self, project_config: ProjectConfig, docker_path: Path):
        template_dir = Path(SCARGO_PGK_PATH, "jinja", "docker")
        BaseGen.__init__(self, template_dir)
        self.docker_path = docker_path
        # List of files to generate (template_path, output_path)
        self._gen_file_list = [
            ("docker-compose.yaml.j2", docker_path / "docker-compose.yaml"),
            ("Dockerfile.j2", docker_path / "Dockerfile"),
            ("devcontainer.json.j2", docker_path / "devcontainer.json"),
        ]
        if project_config.target.family == "stm32":
            self._gen_file_list.extend([("stm32.cfg.j2", docker_path / "stm32.cfg")])
        self.project_config = project_config

    def generate_docker_env(self):
        """Generate dirs and files"""
        custom_docker = ""
        self.create_file_from_template(
            "Dockerfile-custom.j2",
            self.docker_path / "Dockerfile-custom",
            overwrite=False,
        )

        project_root = get_project_root()
        custom_docker_path = project_root / self.project_config.docker_file
        if custom_docker_path.is_file():
            custom_docker = custom_docker_path.read_text()

        logger = get_logger()
        logger.debug(
            "Custom docker file path: %s", custom_docker_path.relative_to(project_root)
        )

        scargo_package_version = self._set_up_package_version()

        for template, output_path in self._gen_file_list:
            self.create_file_from_template(
                template,
                output_path,
                project=self.project_config,
                scargo_package_version=scargo_package_version,
                custom_docker=custom_docker,
            )

    def _set_up_package_version(self) -> str:
        if whl_path := os.getenv("SCARGO_DOCKER_INSTALL_LOCAL"):
            repo_root = Path(__file__).parent.parent.parent
            whl_path = repo_root / whl_path
            shutil.copy(repo_root / whl_path, self.docker_path)
            return whl_path.name
        return f"scargo=={__version__}"


def generate_docker_compose(docker_path: Path, project_config: ProjectConfig):
    docker_compose_template = _DockerComposeTemplate(project_config, docker_path)
    docker_compose_template.generate_docker_env()



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

from pathlib import Path

from scargo.jinja.base_gen import BaseGen
from scargo.scargo_src.global_values import SCARGO_PGK_PATH
from scargo.scargo_src.sc_config import Config
from scargo.scargo_src.utils import get_project_root


class _CMakeTemplate(BaseGen):
    """Cmake template class"""

    def __init__(self):
        template_dir = Path(SCARGO_PGK_PATH, "jinja", "templates")
        BaseGen.__init__(self, template_dir)

    def generate_cmakes(self, config: Config):
        self.create_file_from_template(
            "CMakeLists.txt.j2",
            get_project_root() / "CMakeLists.txt",
            config=config,
        )


def generate_cmake(config: Config):
    cmake_template = _CMakeTemplate()
    cmake_template.generate_cmakes(config)



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

"""Handle docker for project"""
import subprocess
import sys
from pathlib import Path
from typing import Sequence

import docker

from scargo.scargo_src.sc_logger import get_logger
from scargo.scargo_src.sc_src import get_scargo_config_or_exit
from scargo.scargo_src.utils import get_project_root


def scargo_docker_build(docker_opts: Sequence) -> None:
    """
    Build docker

    :param docker_opts: additional docker options
    :raises CalledProcessError: if docker build fail
    """
    logger = get_logger()
    logger.debug("Build docker environment.")

    docker_path = _get_docker_path()
    cmd = " ".join(["docker-compose build", *docker_opts])

    try:
        subprocess.run(cmd, shell=True, cwd=docker_path, check=True)
        logger.info("Initialize docker environment.")
    except subprocess.CalledProcessError:
        logger.error("Build docker fail.")
        sys.exit(1)


def scargo_docker_run(
    docker_opts: Sequence,
    command: str = None,
) -> None:
    """
    Run docker

    :param docker_opts: additional docker options
    :raises CalledProcessError: if docker did not start
    """
    logger = get_logger()
    logger.debug("Run docker environment.")

    docker_path = _get_docker_path()
    project_config_name = _get_project_config().name

    cmd = " ".join(
        filter(
            None,
            ["docker-compose run", *docker_opts, f"{project_config_name}_dev", command],
        )
    )

    try:
        subprocess.run(cmd, shell=True, cwd=docker_path, check=True)
        logger.info("Stop docker environment.")
    except subprocess.CalledProcessError:
        logger.error("Run docker fail.")
        sys.exit(1)


def scargo_docker_exec(docker_opts: Sequence):
    """
    Exec docker

    :param docker_opts: additional docker options
    :raises CalledProcessError: if docker did not start
    """
    logger = get_logger()
    logger.debug("Exec docker environment.")

    image = _get_project_config().docker_image_tag

    if not image:
        logger.error("docker-image-tag not defined in .toml under project section")
        sys.exit(1)

    client = docker.from_env()
    newest_container = client.containers.list(
        limit=1, filters={"ancestor": image, "status": "running"}
    )
    if not newest_container:
        logger.error("No running containers using image `%s` to attach to!", image)
        logger.info("Use scargo docker run to run container.")
        sys.exit(1)

    bash_command = ["bash"]
    cmd = ["docker", "exec"] + docker_opts + [newest_container[0].id] + bash_command
    try:
        subprocess.run(cmd, check=True)
        logger.info("Stop exec docker environment.")
    except subprocess.CalledProcessError:
        logger.error("Exec docker fail.")
        sys.exit(1)


def _get_docker_path():
    project_path = get_project_root()
    # do not rebuild dockers in the docker
    if Path(project_path, ".dockerenv").exists():
        logger = get_logger()
        logger.error("Cannot used docker command inside the docker container.")
        sys.exit(1)
    return Path(project_path, ".devcontainer")


def _get_project_config():
    return get_scargo_config_or_exit().project
